// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.

define(['./loginmain', './logoutmain'], function (login_main, logout_main) {
    return {
        login_main: login_main,
        logout_main: logout_main
    };
});
